@@ -35,6 +35,8 @@
 #include "spdlog/fmt/fmt.h"
 #include "chobo/small_vector.hpp"
 #include "RapMapConfig.hpp"
+#include "nonstd/optional.hpp"
+#include "FastxParser.hpp"
 
 #ifdef RAPMAP_SALMON_SUPPORT
 #include "LibraryFormat.hpp"
@@ -82,8 +84,9 @@ namespace rapmap {
     public:
       bool consistentHits{false};
       bool doChaining{false};
-      int32_t maxSlack{0};
+      float consensusFraction{1.0};
       bool considerMultiPos{false};
+      bool allowDovetail{false};
     };
 
     // Positions are stored in a packed format, where the highest
@@ -211,6 +214,7 @@ namespace rapmap {
         std::atomic<uint64_t> numReads{0};
         std::atomic<uint64_t> tooManyHits{0};
         std::atomic<uint64_t> lastPrint{0};
+        std::atomic<uint64_t> numDovetails{0};
     };
 
     class JFMerKeyHasher{
@@ -431,39 +435,41 @@ namespace rapmap {
       inline double chainScore() const {
         return chainScore_;
       }
-        // Some convenience functions to allow salmon interop
+
+      inline uint32_t transcriptID() const { return tid; }
+      inline double score() const { return score_; }
+      inline void score(double scoreIn) { score_ = scoreIn; }
+      inline int32_t alnScore() const { return alnScore_; }
+      inline void alnScore(int32_t alnScoreIn) { alnScore_ = alnScoreIn; }
+      inline uint32_t fragLength() const { return fragLen; }
+      inline int32_t hitPos() { return std::min(pos, matePos); }
+
+// Some convenience functions to allow salmon interop
 #ifdef RAPMAP_SALMON_SUPPORT
-        inline uint32_t transcriptID() const { return tid; }
-        inline double score() const { return score_; }
-        inline void score(double scoreIn) { score_ = scoreIn; }
-        inline int32_t alnScore() const { return alnScore_; }
-        inline void alnScore(int32_t alnScoreIn) { alnScore_ = alnScoreIn; }
-        inline uint32_t fragLength() const { return fragLen; }
-
-        inline uint32_t fragLengthPedantic(uint32_t txpLen) const {
-            if (mateStatus != rapmap::utils::MateStatus::PAIRED_END_PAIRED
-                or fwd == mateIsFwd) {
-                return 0;
-            }
-            int32_t p1 = fwd ? pos : matePos;
-            int32_t sTxpLen = static_cast<int32_t>(txpLen);
-            p1 = (p1 < 0) ? 0 : p1;
-            p1 = (p1 > sTxpLen) ? sTxpLen : p1;
-            int32_t p2 = fwd ? matePos + mateLen : pos + readLen;
-            p2 = (p2 < 0) ? 0 : p2;
-            p2 = (p2 > sTxpLen) ? sTxpLen : p2;
-
-            return (p1 > p2) ? p1 - p2 : p2 - p1;
+      inline uint32_t fragLengthPedantic(uint32_t txpLen) const {
+        if (mateStatus != rapmap::utils::MateStatus::PAIRED_END_PAIRED
+            or fwd == mateIsFwd) {
+          return 0;
         }
+        int32_t p1 = fwd ? pos : matePos;
+        int32_t sTxpLen = static_cast<int32_t>(txpLen);
+        p1 = (p1 < 0) ? 0 : p1;
+        p1 = (p1 > sTxpLen) ? sTxpLen : p1;
+        int32_t p2 = fwd ? matePos + mateLen : pos + readLen;
+        p2 = (p2 < 0) ? 0 : p2;
+        p2 = (p2 > sTxpLen) ? sTxpLen : p2;
 
-        inline int32_t hitPos() { return std::min(pos, matePos); }
-        double logProb{HUGE_VAL};
-        double logBias{HUGE_VAL};
-        inline LibraryFormat libFormat() { return format; }
-        LibraryFormat format;
+        return (p1 > p2) ? p1 - p2 : p2 - p1;
+      }
+
+      double logProb{HUGE_VAL};
+      double logBias{HUGE_VAL};
+      inline LibraryFormat libFormat() { return format; }
+      LibraryFormat format;
 #endif // RAPMAP_SALMON_SUPPORT
        bool hasMultiPos{false};
        chobo::small_vector<int32_t> allPositions;
+       chobo::small_vector<int32_t> oppositeStrandPositions;
 
         // Only 1 since the mate must have the same tid
         // we won't call *chimeric* alignments here.
@@ -493,6 +499,7 @@ namespace rapmap {
         // If one or both of the reads is a complete match (no mismatch, indels), say what kind.
         FragmentChainStatus chainStatus;
         double chainScore_{std::numeric_limits<double>::lowest()};
+      //int32_t queryOffset{-1};
       //MateStatus completeMatchType{MateStatus::NOTHING};
     };
 
@@ -817,6 +824,10 @@ namespace rapmap {
 
         std::string reverseComplement(std::string& seq);
 
+
+        uint32_t writeUnalignedSingleToStream(fastx_parser::ReadSeq& r, fmt::MemoryWriter& sstream);
+        uint32_t writeUnalignedPairToStream(fastx_parser::ReadPair& r, fmt::MemoryWriter& sstream);
+
         template <typename ReadPairT, typename IndexT>
         uint32_t writeAlignmentsToStream(
                 ReadPairT& r,
@@ -843,7 +854,16 @@ namespace rapmap {
           return MateStatus::PAIRED_END_PAIRED;
         }
 
-        inline void mergeLeftRightHitsFuzzy(
+      enum class MergeResult : uint8_t {
+        HAD_NONE,
+        HAD_EMPTY_INTERSECTION,
+        HAD_CONCORDANT,
+        HAD_DISCORDANT,
+        HAD_ONLY_LEFT,
+        HAD_ONLY_RIGHT,
+      };
+
+        inline MergeResult mergeLeftRightHitsFuzzy(
                 bool leftMatches,
                 bool rightMatches,
                 std::vector<QuasiAlignment>& leftHits,
@@ -855,7 +875,12 @@ namespace rapmap {
                 bool& tooManyHits,
                 HitCounters& hctr) {
 
+          using rapmap::utils::MergeResult;
+          MergeResult mergeRes{MergeResult::HAD_NONE};
+
+          const constexpr int32_t dovetailPenalty = std::numeric_limits<int32_t>::max() / 2;
           bool considerMultiPos = mc.considerMultiPos;
+          bool allowDovetail = mc.allowDovetail;
             if (leftHits.empty()) {
                 if (!leftMatches) {
                     if (!rightHits.empty()) {
@@ -863,6 +888,7 @@ namespace rapmap {
                                 std::make_move_iterator(rightHits.begin()),
                                 std::make_move_iterator(rightHits.end()));
                         hctr.seHits += rightHits.size();
+                        mergeRes = MergeResult::HAD_ONLY_RIGHT;
                     }
                 }
             } else if (rightHits.empty()) {
@@ -872,10 +898,14 @@ namespace rapmap {
                                 std::make_move_iterator(leftHits.begin()),
                                 std::make_move_iterator(leftHits.end()));
                         hctr.seHits += leftHits.size();
+                        mergeRes = MergeResult::HAD_ONLY_LEFT;
                     }
                 }
             } else {
+                bool bestMappingIsDovetail = true;
+                bool hadOppositeStrandMapping = false;
                 constexpr const int32_t signedZero{0};
+                uint32_t sameTxpCount{0};
                 auto leftIt = leftHits.begin();
                 auto leftEnd = leftHits.end();
                 auto leftLen = std::distance(leftIt, leftEnd);
@@ -893,106 +923,189 @@ namespace rapmap {
                             ++leftIt;
                         } else {
                             if (!(rightTxp < leftTxp)) {
+                              ++sameTxpCount;
 
-                              // we start off using the first (or only) positions for the left and right read
-                              auto bestLeftPosIt = leftIt->allPositions.begin();
-                              auto bestRightPosIt = rightIt->allPositions.begin();
+                              // returned tuple is fwPos, rcPos, gapLength
+                              auto findBestHitFWRC = [signedZero, considerMultiPos, allowDovetail, &hadOppositeStrandMapping](
+                                                                       chobo::small_vector<int32_t>& fwdHits,
+                                                                       chobo::small_vector<int32_t>& rcHits,
+                                                                       int32_t fwdReadLen,
+                                                                       bool& bestHitIsDovetail) ->
+                                nonstd::optional<std::tuple<int32_t, int32_t, int32_t>> {
 
-                              // if we are considering multiple positions, and either of these reads appears in
-                              // more than one place, then find the best (smallest gap).
-                              if (considerMultiPos and (leftIt->hasMultiPos or rightIt->hasMultiPos)) {
-                                // NOTE: There is certainly a better way to do this, but current
-                                // algorithm is O(n1 log n2) where
-                                // n1 = leftIt->allPositions.size()
-                                // n2 = leftIt->allPositions.size()
+                                // If either of the position vectors is empty, there can be no valid
+                                // mapping.
+                                if (fwdHits.empty() or rcHits.empty()) {
+                                  bestHitIsDovetail = true;
+                                  return nonstd::nullopt;
+                                }
+                                hadOppositeStrandMapping = true;
 
                                 // Remember the pair of positions that gives us the best gap
                                 // here, a gap of 0 is "optimal".
                                 int32_t bestGap = std::numeric_limits<int32_t>::max();
+                                auto bestFWPosIt = fwdHits.begin();
+                                auto bestRCPosIt = rcHits.begin();
+
+                                // NOTE: Do we need an explicit fast path here?
+                                // if (considerMultiPos and (fwdHits.size() > 1 or rcHits.size() > 1))
+
                                 // Given a left position and a right position, if they produce a better
                                 // gap than the current best, then update the best gap and remember these
                                 // positions that produced it.
-                                auto updateBestGap = [&bestGap, &bestLeftPosIt, &bestRightPosIt, leftIt, rightIt, signedZero](
-                                                                                                 int32_t startRead1,
-                                                                                                 chobo::small_vector<int32_t>::iterator leftPosIt,
-                                                                                                 chobo::small_vector<int32_t>::iterator rightPosIt
-                                                                                                 ) {
-                                                       // The start position for read 2
-                                                       int32_t startRead2 = std::max(*rightPosIt, signedZero);
-                                                       // The gap between the end of the first read and the start of the
-                                                       // second (we take the absolute value so it is always non-negative,
-                                                       // even if they overlap).
-                                                       int32_t gap = (startRead1 < startRead2) ?
-                                                         std::abs(startRead2 - (startRead1 + static_cast<int32_t>(leftIt->readLen))) :
-                                                         std::abs(startRead1 - (startRead2 + static_cast<int32_t>(rightIt->readLen)));
-                                                       if (gap < bestGap) {
-                                                         bestGap = gap;
-                                                         bestLeftPosIt = leftPosIt;
-                                                         bestRightPosIt = rightPosIt;
-                                                       }
-                                                     };
+                                auto updateBestGap = [&bestGap, &bestFWPosIt, &bestRCPosIt,
+                                                      signedZero, fwdReadLen, allowDovetail](
+                                                                                   chobo::small_vector<int32_t>::iterator fwdPosIt,
+                                                                                   chobo::small_vector<int32_t>::iterator rcPosIt
+                                                                                   ) {
+                                  // The gap between the end of the first read and the start of the
+                                  // second (we take the absolute value so it is always non-negative,
+                                  // even if they overlap).
+
+                                  // we expect the rc read to be "downstream" of the fwd read.
+                                  constexpr int32_t maxGap = std::numeric_limits<int32_t>::max();
+
+                                  int32_t gap = maxGap;
+                                  int32_t fwdPos = *fwdPosIt;
+                                  int32_t rcPos = *rcPosIt;
+
+                                  // If the rc read is already downstream of the fwd read, then it's not dovetailed
+                                  // so figure out the gap.
+                                  if (rcPos >= fwdPos) {
+                                    // NOTE: Can think harder about what the best measure of the gap penalty is.
+                                    gap = std::abs(rcPos - (fwdPos + static_cast<int32_t>(fwdReadLen)));
+                                  } else {
+                                    // If the rc read is upstream of the fwd read, then compute the gap with dovetail penalty
+                                    // if dovetails are allowed; otherwise leave it as maxGap.
+                                    gap = (fwdPos - rcPos) + dovetailPenalty;
+                                  }
+
+                                  // if this is the best gap so far
+                                  if (gap < bestGap) {
+                                    bestGap = gap;
+                                    bestFWPosIt = fwdPosIt;
+                                    bestRCPosIt = rcPosIt;
+                                  }
+                                };
 
 
-                                auto rightBeg = rightIt->allPositions.begin();
-                                auto rightEnd = rightIt->allPositions.end();
+                                auto rcBeg = rcHits.begin(); auto rcEnd = rcHits.end();
 
-                                // for every position the left read could start
-                                for (auto pIt = leftIt->allPositions.begin(); pIt != leftIt->allPositions.end(); ++pIt) {
+                                // for every position the forward read could start
+                                for (auto pIt = fwdHits.begin(); pIt != fwdHits.end(); ++pIt) {
                                   auto p1 = *pIt;
-                                  int32_t startRead1 = std::max(p1, signedZero);
 
-                                  // find the closest position for the right read
-                                  auto lbIt = std::lower_bound(rightBeg, rightEnd, p1);
+                                  // find the closest position for the rc read
+                                  auto lbIt = std::lower_bound(rcBeg, rcEnd, p1);
 
-                                  // p1 is greater than every position where the right read can start
-                                  if (lbIt == rightEnd) {
+                                  // p1 is greater than every position where the rc read can start
+                                  if (lbIt == rcEnd) {
                                     auto closestIt = lbIt - 1;
-                                    updateBestGap(startRead1, pIt, closestIt);
-                                  } else if (lbIt == rightBeg) {
-                                    // every position where the right read can start is greater than p1
-                                    updateBestGap(startRead1, pIt, lbIt);
+                                    updateBestGap(pIt, closestIt);
+                                  } else if (lbIt == rcBeg) {
+                                    // every position where the rc read can start is greater than p1
+                                    updateBestGap(pIt, lbIt);
                                   } else {
                                     // check the current element
-                                    updateBestGap(startRead1, pIt, lbIt);
+                                    updateBestGap(pIt, lbIt);
                                     // and the previous
                                     auto prevIt = lbIt - 1;
-                                    updateBestGap(startRead1, pIt, prevIt);
+                                    updateBestGap(pIt, prevIt);
                                   }
                                 }
-                              } // end find best pos
+
+                                // if we had a valid gap, return the best gap
+                                bestHitIsDovetail = (bestGap >= dovetailPenalty);
+                                if (allowDovetail) {
+                                  return (bestGap ==  std::numeric_limits<int32_t>::max()) ? nonstd::nullopt :
+                                  nonstd::optional<std::tuple<int32_t, int32_t, int32_t>>(std::make_tuple(*bestFWPosIt, *bestRCPosIt, bestGap));
+                                } else {
+                                  return (bestGap > dovetailPenalty) ? nonstd::nullopt :
+                                  nonstd::optional<std::tuple<int32_t, int32_t, int32_t>>(std::make_tuple(*bestFWPosIt, *bestRCPosIt, bestGap));
+                                }
+                              };
 
 
+                              // valid pairings have hits on opposite strands
+                              auto& leftFwdHits = (leftIt->fwd) ? leftIt->allPositions : leftIt->oppositeStrandPositions;
+                              auto& leftRCHits  = (leftIt->fwd) ? leftIt->oppositeStrandPositions : leftIt->allPositions;
 
-                              // If we consider only a single position per transcript
-                                int32_t startRead1 = std::max(*bestLeftPosIt, signedZero);
-                                int32_t startRead2 = std::max(*bestRightPosIt, signedZero);
+                              auto& rightFwdHits = (rightIt->fwd) ? rightIt->allPositions : rightIt->oppositeStrandPositions;
+                              auto& rightRCHits  = (rightIt->fwd) ? rightIt->oppositeStrandPositions : rightIt->allPositions;
+
+                              bool bestHitIsDovetailFWRC{false};
+                              bool bestHitIsDovetailRCFW{false};
+                              auto bestFWRC = findBestHitFWRC(leftFwdHits, rightRCHits, static_cast<int32_t>(leftIt->readLen), bestHitIsDovetailFWRC);
+                              auto bestRCFW = findBestHitFWRC(rightFwdHits, leftRCHits, static_cast<int32_t>(rightIt->readLen), bestHitIsDovetailRCFW);
+                              bestMappingIsDovetail = (bestMappingIsDovetail and bestHitIsDovetailFWRC and bestHitIsDovetailRCFW);
+
+                              bool foundValidHit{false};
+                              bool leftFwd{false};
+                              bool rightFwd{false};
+                              int32_t bestGap{std::numeric_limits<int32_t>::max()};
+                              int32_t leftPos = -1, rightPos = -1;
+                              if (bestFWRC){
+                                std::tie(leftPos, rightPos, bestGap) = *bestFWRC;
+                                leftFwd = true; rightFwd = false;
+                                foundValidHit = true;
+                              }
+                              if (bestRCFW) {
+                                int32_t fwPos, rcPos, bestGapRCFW;
+                                std::tie(fwPos, rcPos, bestGapRCFW) = *bestRCFW;
+                                if (bestGapRCFW < bestGap) {
+                                  leftPos = rcPos;
+                                  rightPos = fwPos;
+                                  leftFwd = false; rightFwd = true;
+                                }
+                                foundValidHit = true;
+                              }
+
+                              if (foundValidHit) {
+                                // If we consider only a single position per transcript
+                                int32_t startRead1 = std::max(leftPos, signedZero);
+                                int32_t startRead2 = std::max(rightPos, signedZero);
                                 bool read1First{(startRead1 < startRead2)};
                                 int32_t fragStartPos = read1First ? startRead1 : startRead2;
                                 int32_t fragEndPos = read1First ?
-                                    (startRead2 + rightIt->readLen) : (startRead1 + leftIt->readLen);
+                                  (startRead2 + rightIt->readLen) : (startRead1 + leftIt->readLen);
                                 uint32_t fragLen = fragEndPos - fragStartPos;
                                 jointHits.emplace_back(leftTxp,
-                                        *bestLeftPosIt,
-                                        leftIt->fwd,
-                                        leftIt->readLen,
-                                        fragLen, true);
+                                                       leftPos,
+                                                       leftFwd,
+                                                       leftIt->readLen,
+                                                       fragLen, true);
                                 // Fill in the mate info
                                 auto& qaln = jointHits.back();
                                 qaln.mateLen = rightIt->readLen;
-                                qaln.matePos = *bestRightPosIt;
-                                qaln.mateIsFwd = rightIt->fwd;
+                                qaln.matePos = rightPos;
+                                qaln.mateIsFwd = rightFwd;
                                 jointHits.back().mateStatus = MateStatus::PAIRED_END_PAIRED;
                                 jointHits.back().chainStatus = FragmentChainStatus(leftIt->chainStatus.getLeft(), rightIt->chainStatus.getRight());
-                                //jointHits.back().completeMatchType = mergeMatchType(leftIt->completeMatchType, rightIt->completeMatchType);
                                 ++numHits;
+                                mergeRes = MergeResult::HAD_CONCORDANT;
                                 if (numHits > maxNumHits) { tooManyHits = true; break; }
-                                ++leftIt;
-                            }
+                              }
+                              ++leftIt;
+
+                            } // END if (!(rightTxp < leftTxp))
+
                             ++rightIt;
                         }
                     }
+                    //if (triedHit and jointHits.empty()) { tooManyHits = true;}
                 }
+                // If we had a potentially valid mapping (hits from different strands), but the
+                // (best) only mappings for this fragment were dovetailed, then increment the
+                // numDovetails counter here.
+                if (bestMappingIsDovetail and hadOppositeStrandMapping) { ++hctr.numDovetails; }
+
                 if (tooManyHits) { jointHits.clear(); ++hctr.tooManyHits; }
+
+                if (mergeRes == MergeResult::HAD_NONE) {
+                  // If we had hits on the same transcript, but our status isn't concordant,
+                  // then we had discordant hits, otherwise, we had a null intersection.
+                  mergeRes = (sameTxpCount > 0) ? MergeResult::HAD_DISCORDANT : MergeResult::HAD_EMPTY_INTERSECTION;
+                }
             }
 
             // If we had proper paired hits
@@ -1000,6 +1113,8 @@ namespace rapmap {
                 hctr.peHits += jointHits.size();
                 //orphanStatus = 0;
             }
+
+            return mergeRes;
         }
 
         inline void mergeLeftRightHits(
@@ -1087,7 +1202,7 @@ namespace rapmap {
     template <typename Archive>
     void save(Archive& archive, const my_mer& mer);
 
-    template <typename Archive>
+    teplate <typename Archive>
     void load(Archive& archive, my_mer& mer);
     */
     }
