@@ -61,8 +61,8 @@ public:
   void setStrictCheck(bool sc) { strictCheck_ = sc; }
 
   /** Get/Set usage of MMP chain scoring **/
-  void enableChainScoring() { doChaining_ = true; }
-  void disableChainScoring() { doChaining_ = false; }
+  void enableChainScoring() { doChaining_ = true; strictCheckFraction_ = 0.65; }
+  void disableChainScoring() { doChaining_ = false; strictCheckFraction_ = 1.0; }
   bool getChainScoring() const { return doChaining_; }
 
   /** Get/Set the "mohsen number" that is used to limit the
@@ -281,11 +281,15 @@ public:
       // If we're computing coverage, then we can make use of that info here
       //useCoverageCheck = false;
       if (useCoverageCheck) {
-        if (fwdCov > rcCov) {
-          rcSAInts.clear();
-        } else if (rcCov > fwdCov) {
+        auto maxCov = std::max(fwdCov, rcCov);
+        int32_t requiredNumHits = (strictCheckFraction_ < 1.0) ?
+          std::max(static_cast<int32_t>(1), static_cast<int32_t>(std::floor(maxCov * strictCheckFraction_))) : maxCov;
+        if (static_cast<int32_t>(fwdCov) < requiredNumHits) {
           fwdSAInts.clear();
         }
+        if (static_cast<int32_t>(rcCov) < requiredNumHits) {
+          rcSAInts.clear();
+        }
       } else { // use the k-mer "spot check"
         // The first two conditions shouldn't happen
         // but I'm just being paranoid here
@@ -684,6 +688,7 @@ private:
   bool strictCheck_;
   bool doChaining_;
   int32_t maxMMPExtension_{7};
+  double strictCheckFraction_{1.0};
   std::string rcBuffer_;
 };
 
